#include "pch.h"
#include "BouncingBallRenderer.h"

using namespace D2D1;
using namespace DirectX;
using namespace Microsoft::WRL;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Core;

static const std::array<ColorF, 7> BallColors = 
{
    ColorF::Orchid,
    ColorF::DeepPink,
    ColorF::Red,
    ColorF::DarkOrange,
    ColorF::Gold,
    ColorF::GreenYellow,
    ColorF::Aqua
};

BouncingBallRenderer::BouncingBallRenderer() :
    m_renderNeeded(true)
{
    m_ballPosition.dX = BALL_DELTA;
    m_ballPosition.dY = BALL_DELTA;
    m_ballPosition.width = 32;
    m_ballPosition.height = 32;
}

void BouncingBallRenderer::CreateDeviceIndependentResources()
{
    DirectXBase::CreateDeviceIndependentResources();
}

void BouncingBallRenderer::CreateDeviceResources()
{
    DirectXBase::CreateDeviceResources();

    DX::ThrowIfFailed(
        m_d2dContext->CreateSolidColorBrush(
        ColorF(BallColors[m_ballColorIndex]),
        &m_blackBrush
        )
        );
}

void BouncingBallRenderer::CreateWindowSizeDependentResources()
{
    DirectXBase::CreateWindowSizeDependentResources();

    // Add code to create window size dependent objects here.
}

void BouncingBallRenderer::Update(float timeTotal, float timeDelta)
{
    (void) timeTotal; // Unused parameter.
    (void) timeDelta; // Unused parameter.

    // Add code to update time dependent objects here.
    const auto windowWidth = m_d3dRenderTargetSize.Width;
    const auto windowHeight = m_d3dRenderTargetSize.Height;

    m_ballPosition.X += m_ballPosition.dX;
    m_ballPosition.Y += m_ballPosition.dY;

    if (m_ballPosition.X < m_ballPosition.width)
    {
        m_ballPosition.X = m_ballPosition.width;
        m_ballPosition.dX = BALL_DELTA;
    }
    else if (m_ballPosition.X + m_ballPosition.width > static_cast<int>(windowWidth))
    {
        m_ballPosition.X = static_cast<int>(windowWidth) - m_ballPosition.width;
        m_ballPosition.dX = -BALL_DELTA;
    }

    if (m_ballPosition.Y < m_ballPosition.height)
    {
        m_ballPosition.Y = m_ballPosition.height;
        m_ballPosition.dY = BALL_DELTA;
    }
    else if (m_ballPosition.Y + m_ballPosition.height > static_cast<int>(windowHeight))
    {
        m_ballPosition.Y = static_cast<int>(windowHeight) - m_ballPosition.height;
        m_ballPosition.dY = -BALL_DELTA;
    }
}

void BouncingBallRenderer::Render()
{
    m_d2dContext->BeginDraw();

    m_d2dContext->Clear(ColorF(ColorF::CornflowerBlue));

    m_d2dContext->FillEllipse(
        Ellipse(Point2F(static_cast<float>( m_ballPosition.X ), 
        static_cast<float>( m_ballPosition.Y)), 
        static_cast<float>( m_ballPosition.width), 
        static_cast<float>( m_ballPosition.height)),
        m_blackBrush.Get());

    // Ignore D2DERR_RECREATE_TARGET. This error indicates that the device
    // is lost. It will be handled during the next call to Present.
    const auto hr = m_d2dContext->EndDraw();
    if (hr != D2DERR_RECREATE_TARGET)
    {
        DX::ThrowIfFailed(hr);
    }

    m_renderNeeded = false;
}


void BouncingBallRenderer::BallColorNext()
{
    m_ballColorIndex++;
    if (m_ballColorIndex >= static_cast<int>(BallColors.size()))
    {
        m_ballColorIndex = 0;
    }

    DX::ThrowIfFailed(
        m_d2dContext->CreateSolidColorBrush(
        ColorF(BallColors[m_ballColorIndex]),
        &m_blackBrush
        )
        );
}

void BouncingBallRenderer::BallColorPrevious()
{
    m_ballColorIndex--;
    if (m_ballColorIndex < 0)
    {
        m_ballColorIndex = static_cast<int>(BallColors.size()) - 1;
    }

    DX::ThrowIfFailed(
        m_d2dContext->CreateSolidColorBrush(
        ColorF(BallColors[m_ballColorIndex]),
        &m_blackBrush
        )
        );
}

void BouncingBallRenderer::SaveInternalState(IPropertySet^ state)
{
    if (state->HasKey("m_ballColorIndex"))
    {
        state->Remove("m_ballColorIndex");
    }

    state->Insert("m_ballColorIndex", PropertyValue::CreateInt32(m_ballColorIndex));
}

void BouncingBallRenderer::LoadInternalState(IPropertySet^ state)
{
    if (state->HasKey("m_ballColorIndex"))
    {
        m_ballColorIndex = safe_cast<IPropertyValue^>(state->Lookup("m_ballColorIndex"))->GetInt32();
    }
}
